%-------------------------------------------------------------------------%
% Background
% Band gap comparison A (focus on shear)
% Created by Lei, Tue, 2017
%-------------------------------------------------------------------------%

clear,clc

%% Cruciform resonantor configuration
mu = 1;
W = 20e-3;                           % width
rho = 2e3;                           % density

r = 0.15 * W;                        % radius
R = 0.25 * W;

M = rho * (W^2 - pi*R^2);            % matrix mass

% comparison 1 (fixed rrho = 10)
rK = [0.001 0.002 0.005 0.01 0.02 0.05 0.1 0.2];
Np = 20;                            % interpolation
method = 'spline';
rK_p = 10.^(linspace(min(log10(rK)),max(log10(rK)),Np));
rms = mu * 10 * pi*r^2 / (W^2 - pi*R^2);
ms = rms*M;                          % inclusion mass

OmegaUset1 = [2.8133e+04
              2.8143e+04
              2.8170e+04
              2.8214e+04
              2.8302e+04
              2.8552e+04
              2.8929e+04
              2.9567e+04];
OmegaUset1_p = [interp1(rK,OmegaUset1,rK_p,method)]';
OmegaUset1 = OmegaUset1_p; 

OmegaRset1 = [668.6362  668.8239
              944.6905  945.2209
              1.4894e3    1.4915e3
              2.0964e3    2.1022e3
              2.9368e3    2.9533e3
              4.5160e3    4.5789e3
              6.1078e3    6.2758e3
              7.9539e3    8.3732e3
             ];                    % resonant frequency, 1: HM; 2: BA
OmegaRset1_p = [interp1(rK,OmegaRset1(:,1),rK_p,method); interp1(rK,OmegaRset1(:,2),rK_p,method)]';         
OmegaRset1 = OmegaRset1_p; 
         
Mtotset1 = [1.2110
            1.2110
            1.2110
            1.2110
            1.2110
            1.2110
            1.2110
            1.2110];                % static mass, rhoS(1,1)*V
Mtotset1_p = [interp1(rK,Mtotset1,rK_p,method)]';
Mtotset1 = Mtotset1_p;     

Bandset1_BA = [ 668.8239  916.3144
                0.9452e3    1.2953e3
                1.4915e3    2.0454e3
                2.1022e3    2.8866e3
                2.9533e3    4.0650e3
                4.5789e3    6.3474e3
                6.2758e3    8.7965e3
                0.8373e4    1.1970e4
                ]; % [Bound1C Bound2C]
Bandset1_BA_p = [interp1(rK,Bandset1_BA(:,1),rK_p,method); interp1(rK,Bandset1_BA(:,2),rK_p,method)]';       
Bandset1_BA = Bandset1_BA_p;
Bandset1_BA(:,1) = Bandset1_BA(:,1)./ OmegaRset1(:,2); % normalized by accurate wr
Bandset1_BA(:,2) = Bandset1_BA(:,2)./ OmegaRset1(:,2);
W1_BA = Bandset1_BA(:,2) - Bandset1_BA(:,1);

Bandset1_HM = [ 668.2853  916.3899
                0.9441e3    1.2955e3
                1.4875e3    2.0455e3
                2.0910e3    2.8868e3
                2.9219e3    4.0651e3
                4.4588e3    6.3477e3
                5.9540e3    8.7972e3
                0.7575e4    1.1971e4
                ];
Bandset1_HM_p = [interp1(rK,Bandset1_HM(:,1),rK_p,method); interp1(rK,Bandset1_HM(:,2),rK_p,method)]';      
Bandset1_HM = Bandset1_HM_p;
Bandset1_HM(:,1) = Bandset1_HM(:,1)./ OmegaRset1(:,1);
Bandset1_HM(:,2) = Bandset1_HM(:,2)./ OmegaRset1(:,1);
W1_HM = Bandset1_HM(:,2) - Bandset1_HM(:,1);

rK = rK_p;

% comparison 2 (fixed rK = 0.01)
rrho = [0.25 0.5 1 2 4 6 8 10 15 20];
rm = mu * rrho * pi*r^2 / (W^2 - pi*R^2);
m = rm * M;

fm = m./(m+M) * 100;

Np = 25;                            % interpolation
method = 'pchip';
% fm_p = [linspace(fm(1),fm(5),13) linspace(fm(5),fm(end),12)];
fm_p = linspace(min(fm),max(fm),Np);

OmegaUset2 = [2.8214e+04
              2.8214e+04
              2.8214e+04
              2.8214e+04
              2.8214e+04
              2.8214e+04
              2.8214e+04
              2.8214e+04
              2.8214e+04
              2.8214e+04
              ];
OmegaUset2_p = [interp1(fm,OmegaUset2,fm_p,method)]';
OmegaUset2 = OmegaUset2_p; 


OmegaRset2 = [1.3220e4    1.3290e4
              9.3645e3    9.3995e3
              6.6260e3    6.6472e3
              4.6866e3    4.7005e3
              3.3144e3    3.3239e3
              2.7063e3    2.7140e3
              2.3438e3    2.3504e3
              2.0964e3    2.1022e3
              1.7117e3    1.7165e3
              1.4824e3    1.4865e3
             ]; 
OmegaRset2_p = [interp1(fm,OmegaRset2(:,1),fm_p,method); interp1(fm,OmegaRset2(:,2),fm_p,method)]';         
OmegaRset2 = OmegaRset2_p; 

Mtotset2 = [0.6596
            0.6738
            0.7020
            0.7586
            0.8717
            0.9848
            1.0979
            1.2110
            1.4937
            1.7765]; 
Mtotset2_p = [interp1(fm,Mtotset2,fm_p,method)]';
Mtotset2 = Mtotset2_p; 

Bandset2_BA = [1.3290e4    1.3473e4
               9.3995e3    9.6287e3
               6.6472e3    6.9500e3
               4.7005e3    5.1085e3
               3.3239e3    3.8722e3
               2.7140e3    3.3605e3
               2.3504e3    3.0729e3
               2.1022e3    2.8866e3
               1.7165e3    2.6176e3
               1.4865e3    2.4722e3
               ];
Bandset2_BA_p = [interp1(fm,Bandset2_BA(:,1),fm_p,method); interp1(fm,Bandset2_BA(:,2),fm_p,method)]';      
Bandset2_BA = Bandset2_BA_p;
Bandset2_BA(:,1) = Bandset2_BA(:,1)./ OmegaRset2(:,2);
Bandset2_BA(:,2) = Bandset2_BA(:,2)./ OmegaRset2(:,2);  

W2_BA = Bandset2_BA(:,2) - Bandset2_BA(:,1);

Bandset2_HM = [1.3079e4    1.3473e4
               9.3216e3    9.6289e3
               6.6043e3    6.9503e3
               4.6734e3    5.1087e3
               3.3057e3    3.8724e3
               2.6993e3    3.3606e3
               2.3378e3    3.0731e3
               2.0910e3    2.8868e3
               1.7074e3    2.6177e3
               1.4786e3    2.4722e3
               ];
Bandset2_HM_p = [interp1(fm,Bandset2_HM(:,1),fm_p,method); interp1(fm,Bandset2_HM(:,2),fm_p,method)]';      
Bandset2_HM = Bandset2_HM_p;
Bandset2_HM(:,1) = Bandset2_HM(:,1)./ OmegaRset2(:,1);
Bandset2_HM(:,2) = Bandset2_HM(:,2)./ OmegaRset2(:,1);  

W2_HM = Bandset2_HM(:,2) - Bandset2_HM(:,1);

fm = fm_p;


%% Plots

Dev1 = abs(W1_BA - W1_HM)./W1_BA * 100;      % deviation (>10% after 15th)
Dev2 = abs(W2_BA - W2_HM)./W2_BA * 100;      % deviation (>10% before 5th)

% comparison 1
figure
subplot(1,2,1)
hold on

[AX,H1,H2]=plotyy(rK,W1_BA,rK,OmegaUset1./OmegaRset1(:,1));
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);

plot(rK,W1_HM,'-ksquare','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',10)

set(AX(1),'xscale', 'log','FontSize',16,'FontName','Times New Roman')
set(AX(2),'xscale', 'log','FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','K/E','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
legend('W_b/\omega_{res}-BA','W_b/\omega_{res}-HM','\mu_{\omega}')

subplot(1,2,2)
hold on
[AX,H1,H2]=plotyy(fm,W2_BA,fm,OmegaUset2./OmegaRset2(:,1));
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);

plot(fm,W2_HM,'-ksquare','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',10)

set(AX(1),'FontSize',16,'FontName','Times New Roman')
set(AX(2),'FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','\mu_\rho [%]','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
set(gcf,'Units','centimeters','Position',[5 0 22 15])

