%-------------------------------------------------------------------------%
% Background
% Band gap comparison B (focus on compression)
% Created by Lei, Tue, 2017
%-------------------------------------------------------------------------%

clear,clc
%% Herringboned resonantor configuration
mu = 2;
W = 20e-3;                           % width
rho = 2e3;                           % density

r = 0.05 * W;                        % radius
R = 0.25 * W;

M = rho * (W^2 - pi*R^2);            % matrix mass

% comparison 1 (fixed rrho = 20)
rK = [0.001 0.002 0.005 0.01 0.02 0.05 0.1 0.2];
Np = 25;                            % interpolation
method = 'spline';
rK_p = 10.^(linspace(min(log10(rK)),max(log10(rK)),Np));
rms = mu * 20 * pi*r^2 / (W^2 - pi*R^2);
ms = rms*M;

OmegaUset1 = [2.0296e4
              2.0296e4
              2.0296e4
              2.0296e4
              2.0296e4
              2.0296e4
              2.0296e4
              2.0296e4];
OmegaUset1_p = [interp1(rK,OmegaUset1,rK_p,method)]';
OmegaUset1 = OmegaUset1_p; 

OmegaRset1 = [1.4129e3    1.4084e3
              1.9887e3    1.9762e3
              3.1005e3    3.0537e3
              4.2869e3    4.1650e3
              5.8112e3    5.5159e3
              8.2382e3    7.4457e3
              1.0113e4    0.8730e4
              1.1701e4    0.9677e4
             ];                    % resonant frequency, 1: HM; 2: BA
OmegaRset1_p = [interp1(rK,OmegaRset1(:,1),rK_p,method); interp1(rK,OmegaRset1(:,2),rK_p,method)]';         
OmegaRset1 = OmegaRset1_p; 
         
Mtotset1 = [0.8968
            0.8968
            0.8968
            0.8968
            0.8968
            0.8968
            0.8968
            0.8968];                % static mass, rhoS(1,1)*V
Mtotset1_p = [interp1(rK,Mtotset1,rK_p,method)]';
Mtotset1 = Mtotset1_p;     

Bandset1_BA = [1.4099e3 1.4129e3
               1.9802e3 1.9887e3
               3.0666e3 3.1006e3
               4.1960e3 4.2871e3
               5.5798e3 5.8115e3
               7.5728e3 8.2386e3
               8.9021e3 1.0113e4
               9.8817e3 1.1701e4
                ]; % [Bound1C Bound2C]
Bandset1_BA_p = [interp1(rK,Bandset1_BA(:,1),rK_p,method); interp1(rK,Bandset1_BA(:,2),rK_p,method)]';       
Bandset1_BA = Bandset1_BA_p;
Bandset1_BA(:,1) = Bandset1_BA(:,1)./ OmegaRset1(:,2); % normalized by accurate wr
Bandset1_BA(:,2) = Bandset1_BA(:,2)./ OmegaRset1(:,2);
W1_BA = Bandset1_BA(:,2) - Bandset1_BA(:,1);

Bandset1_HM = [1.4099e3 1.4129e3
               1.9806e3 1.9887e3
               3.0706e3 3.1006e3
               4.2091e3 4.2871e3
               5.6185e3 5.8115e3
               7.6993e3 8.2386e3
               9.1450e3 1.0113e4
               1.0247e4 1.1701e4
                ];
Bandset1_HM_p = [interp1(rK,Bandset1_HM(:,1),rK_p,method); interp1(rK,Bandset1_HM(:,2),rK_p,method)]';      
Bandset1_HM = Bandset1_HM_p;
Bandset1_HM(:,1) = Bandset1_HM(:,1)./ OmegaRset1(:,1);
Bandset1_HM(:,2) = Bandset1_HM(:,2)./ OmegaRset1(:,1);
W1_HM = Bandset1_HM(:,2) - Bandset1_HM(:,1);

rK = rK_p;

% comparison 2 (fixed rK = 0.01)
rrho = [2 4 6 10 15 20 30 50];
rm = mu * rrho * pi*r^2 / (W^2 - pi*R^2);
m = rm * M;
fm = m./(m+M) * 100;

Np = 25;                            % interpolation
method = 'pchip';
fm_p = [linspace(fm(1),fm(5),7) linspace(fm(5),fm(end),8)];
% fm_p = linspace(min(fm),max(fm),Np);

OmegaUset2 = [3.9645e4
              3.4601e4
              3.1094e4
              2.6422e4
              2.2762e4
              2.0296e4
              1.7092e4
              1.3592e4
              ];
OmegaUset2_p = [interp1(fm,OmegaUset2,fm_p,method)]';
OmegaUset2 = OmegaUset2_p; 


OmegaRset2 = [1.3553e4    1.3132e4
              9.5848e3    9.3013e3
              7.8263e3    7.5986e3
              6.0624e3    5.8884e3
              4.9500e3    4.8088e3
              4.2869e3    4.1650e3
              3.5002e3    3.4011e3
              2.7113e3    2.6347e3
             ]; 
OmegaRset2_p = [interp1(fm,OmegaRset2(:,1),fm_p,method); interp1(fm,OmegaRset2(:,2),fm_p,method)]';         
OmegaRset2 = OmegaRset2_p; 

Mtotset2 = [0.6706
            0.6957
            0.7209
            0.7711
            0.8340
            0.8968
            1.0225
            1.2738
            ]; 
Mtotset2_p = [interp1(fm,Mtotset2,fm_p,method)]';
Mtotset2 = Mtotset2_p; 

Bandset2_BA = [1.3160e4 1.3558e4
               9.3401e3 9.5849e3
               7.6382e3 7.8266e3
               5.9246e3 6.0626e3
               4.8426e3 4.9504e3
               4.1960e3 4.2871e3
               3.4278e3 3.5004e3
               2.6561e3 2.7114e3
               ];
Bandset2_BA_p = [interp1(fm,Bandset2_BA(:,1),fm_p,method); interp1(fm,Bandset2_BA(:,2),fm_p,method)]';       
Bandset2_BA = Bandset2_BA_p;
Bandset2_BA(:,1) = Bandset2_BA(:,1)./ OmegaRset2(:,2);
Bandset2_BA(:,2) = Bandset2_BA(:,2)./ OmegaRset2(:,2);  

W2_BA = Bandset2_BA(:,2) - Bandset2_BA(:,1);

Bandset2_HM = [1.3265e4 1.3558e4
               9.3942e3 9.5849e3
               7.6755e3 7.8266e3
               5.9501e3 6.0626e3
               4.8599e3 4.9504e3
               4.2091e3 4.2871e3
               3.4372e3 3.5004e3
               2.6626e3 2.7114e3
               ];
Bandset2_HM_p = [interp1(fm,Bandset2_HM(:,1),fm_p,method); interp1(fm,Bandset2_HM(:,2),fm_p,method)]';       
Bandset2_HM = Bandset2_HM_p;
Bandset2_HM(:,1) = Bandset2_HM(:,1)./ OmegaRset2(:,1);
Bandset2_HM(:,2) = Bandset2_HM(:,2)./ OmegaRset2(:,1);  

W2_HM = Bandset2_HM(:,2) - Bandset2_HM(:,1);
   
% mu_D versus alpha (K/E=0.01, rrho = [2 4 6 10 15 20 30 50])
method = 'spline';
Alpha = [11.3, 21.8 31.0]; % included angle, deg
Omega_R1 = [1.2047e4
            8.5211e3
            6.9582e3
            5.3902e3
            4.4013e3
            3.8117e3
            3.1123e3
            2.4108e3];
Omega_R1=[interp1(fm,Omega_R1,fm_p,method)]';

mu_D1 = [0.6456
         0.7846
         0.8453
         0.9011
         0.9318
         0.9480
         0.9647
         0.9785]*100;
rmu_D1=[interp1(fm,mu_D1,fm_p,method)]'./fm_p';   % mu_D/mu_rho

Omega_R2 = [1.3553e4
            9.5848e3
            7.8263e3
            6.0624e3
            4.9500e3
            4.2869e3
            3.5002e3
            2.7113e3];   
Omega_R2=[interp1(fm,Omega_R2,fm_p,method)]';

mu_D2 = [0.3129
         0.4766
         0.5773
         0.6948
         0.7735
         0.8199
         0.8723
         0.9192]*100;
rmu_D2=[interp1(fm,mu_D2,fm_p,method)]'./fm_p';

Omega_R3 = [1.3900e4
            9.8294e3
            8.0258e3
            6.2169e3
            5.0761e3
            4.3961e3
            3.5894e3
            2.7803e3];   
Omega_R3=[interp1(fm,Omega_R3,fm_p,method)]';

mu_D3 = [0.1684
         0.2882
         0.3778
         0.5030
         0.6028
         0.6693
         0.7522
         0.8350]*100;
rmu_D3 = [interp1(fm,mu_D3,fm_p,method)]'./fm_p';

Omega_R = [Omega_R1 Omega_R2 Omega_R3]/1000;
rmu_D = [rmu_D1 rmu_D2 rmu_D3];
fm = fm_p;

%% Plots

Dev1 = abs(W1_BA - W1_HM)./W1_BA * 100;      % deviation (>20% after 13th)
Dev2 = abs(W2_BA - W2_HM)./W2_BA * 100;      % deviation (>20% before 6th)

% comparison 1
figure
subplot(1,3,1)
hold on

[AX,H1,H2]=plotyy(rK,W1_BA,rK,OmegaUset1./OmegaRset1(:,1));
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);

plot(rK,W1_HM,'-ksquare','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',10)

set(AX(1),'xscale', 'log','FontSize',16,'FontName','Times New Roman')
set(AX(2),'xscale', 'log','FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','K/E','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
legend('W_b/\omega_{res}-BA','W_b/\omega_{res}-HM','\mu_{\omega}')

subplot(1,3,2)
hold on
[AX,H1,H2]=plotyy(fm,W2_BA,fm,OmegaUset2./OmegaRset2(:,1));
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);

plot(fm,W2_HM,'-ksquare','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',10)

set(AX(1),'FontSize',16,'FontName','Times New Roman')
set(AX(2),'FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','\mu_\rho [%]','FontName','Times New Roman','FontWeight','Bold','FontSize',16);

subplot(1,3,3)
hold on
[AX,H1,H2]=plotyy(fm,rmu_D,fm,Omega_R);
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);
set(AX(1),'FontSize',16,'FontName','Times New Roman')
set(AX(2),'FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','\mu_\rho [%]','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
set(get(AX(1),'Ylabel'),'String','\mu_D/\mu_\rho','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
set(get(AX(2),'Ylabel'),'String','\omega_{res} [kHz]','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
legend('\alpha=11.3^o','\alpha=21.8^o','\alpha=31.0^o')
set(gcf,'Units','centimeters','Position',[5 0 35 15])
