%-------------------------------------------------------------------------%
% Background
% Band gap comparison C (focus on compression)
% Created by Lei, Tue, 2017
%-------------------------------------------------------------------------%

clear,clc

%% Torsion resonantor configuration

% comparison 1 (fixed rrho = 10)
mu = 1;
W = 20e-3;                           % width
rho = 2e3;                           % density

r = 0.15 * W;                        % radius
R = 0.25 * W;

M = rho * (W^2 - pi*R^2);            % matrix mass

rK = [0.005 0.0075 0.01 0.015 0.03 0.05 0.075 0.1];
Np = 25;                            % interpolation
method = 'spline';
rK_p = 10.^(linspace(min(log10(rK)),max(log10(rK)),Np));



rms = mu * 10 * pi*r^2 / (W^2 - pi*R^2);
ms = rms*M;

OmegaUset1 = [4.2493e4
              4.2504e4
              4.2516e4
              4.2539e4
              4.2607e4
              4.2691e4
              4.2789e4
              4.2879e4];
OmegaUset1_p = [interp1(rK,OmegaUset1,rK_p,method)]';
OmegaUset1 = OmegaUset1_p; 

OmegaRset1 = [3.6163e3    3.6121e3
              4.4209e3    4.4133e3
              5.0955e3    5.0838e3
              6.2180e3    6.1966e3
              8.6992e3    8.6394e3
              1.1074e4    1.0948e4
              1.3333e4    1.3106e4
              1.5142e4    1.4802e4
             ];                    % resonant frequency, 1: HM; 2: BA
OmegaRset1_p = [interp1(rK,OmegaRset1(:,1),rK_p,method); interp1(rK,OmegaRset1(:,2),rK_p,method)]';         
OmegaRset1 = OmegaRset1_p; 
         
Mtotset1 = [1.2110
            1.2110
            1.2110
            1.2110
            1.2110
            1.2110
            1.2110
            1.2110];                % static mass, rhoS(1,1)*V
Mtotset1_p = [interp1(rK,Mtotset1,rK_p,method)]';
Mtotset1 = Mtotset1_p;     

Bandset1_BA = [3.6121e3 3.6163e3
               4.4135e3 4.4209e3
               5.0841e3 5.0955e3
               6.1967e3 6.2180e3
               8.6394e3 8.6994e3
               1.0948e4 1.1074e4
               1.3106e4 1.3333e4
               1.4802e4 1.5142e4
                ]; % [Bound1C Bound2C]
Bandset1_BA_p = [interp1(rK,Bandset1_BA(:,1),rK_p,method); interp1(rK,Bandset1_BA(:,2),rK_p,method)]';       
Bandset1_BA = Bandset1_BA_p;
Bandset1_BA(:,1) = Bandset1_BA(:,1)./ OmegaRset1(:,2); % normalized by accurate wr
Bandset1_BA(:,2) = Bandset1_BA(:,2)./ OmegaRset1(:,2);
W1_BA = Bandset1_BA(:,2) - Bandset1_BA(:,1);

Bandset1_HM = [3.6124e3 3.6163e3
               4.4142e3 4.4209e3
               5.0855e3 5.0955e3
               6.1998e3 6.2180e3
               8.6483e3 8.6994e3
               1.0975e4 1.1074e4
               1.3159e4 1.3333e4
               1.4894e4 1.5142e4
                ];
Bandset1_HM_p = [interp1(rK,Bandset1_HM(:,1),rK_p,method); interp1(rK,Bandset1_HM(:,2),rK_p,method)]';      
Bandset1_HM = Bandset1_HM_p;
Bandset1_HM(:,1) = Bandset1_HM(:,1)./ OmegaRset1(:,1);
Bandset1_HM(:,2) = Bandset1_HM(:,2)./ OmegaRset1(:,1);
W1_HM = Bandset1_HM(:,2) - Bandset1_HM(:,1);

rK = rK_p;

% comparison 2 (fixed rK = 0.02)
rrho = [3 5 6 8 11 15 17 20];
rm = mu * rrho * pi*r^2 / (W^2 - pi*R^2);
m = rm * M;
fm = m./(m+M) * 100;

Np = 15;                            % interpolation
method = 'pchip';
fm_p = linspace(min(fm),max(fm),Np);


OmegaUset2 = [
              4.6113e4
              4.5009e4
              4.4486e4
              4.3492e4
              4.2119e4
              4.0476e4
              3.9723e4
              3.8669e4
              ];
OmegaUset2_p = [interp1(fm,OmegaUset2,fm_p,method)]';
OmegaUset2 = OmegaUset2_p; 

OmegaRset2 = [
              1.3061e4    1.2994e4
              1.0117e4    1.0069e4
              9.2356e3    9.1920e3
              7.9983e3    7.9612e3
              6.8211e3    6.7899e3
              5.8412e3    5.8148e3
              5.4869e3    5.4621e3
              5.0587e3    5.0359e3
             ]; 
OmegaRset2_p = [interp1(fm,OmegaRset2(:,1),fm_p,method); interp1(fm,OmegaRset2(:,2),fm_p,method)]';         
OmegaRset2 = OmegaRset2_p; 

Mtotset2 = [0.8126
            0.9257
            0.9822
            1.0953
            1.2650
            1.4912
            1.6068
            1.7739
            ]; 
Mtotset2_p = [interp1(fm,Mtotset2,fm_p,method)]';
Mtotset2 = Mtotset2_p; 

Bandset2_BA = [1.2989e4 1.3061e4
               1.0067e4 1.0117e4
               9.1919e3 9.2357e3
               7.9614e3 7.9985e3
               
               6.7901e3 6.8212e3
               5.8151e3 5.8414e3
               5.4627e3 5.4871e3
               5.0365e3 5.0587e3
               ];
Bandset2_BA_p = [interp1(fm,Bandset2_BA(:,1),fm_p,method); interp1(fm,Bandset2_BA(:,2),fm_p,method)]';       
Bandset2_BA = Bandset2_BA_p;
Bandset2_BA(:,1) = Bandset2_BA(:,1)./ OmegaRset2(:,2);
Bandset2_BA(:,2) = Bandset2_BA(:,2)./ OmegaRset2(:,2);  

W2_BA = Bandset2_BA(:,2) - Bandset2_BA(:,1);

Bandset2_HM = [1.3005e4 1.3061e4
               1.0076e4 1.0117e4
               9.1987e3 9.2357e3
               7.9669e3 7.9985e3
               6.7945e3 6.8212e3
               5.8188e3 5.8414e3
               5.4660e3 5.4871e3
               5.0395e3 5.0587e3
               ];
Bandset2_HM_p = [interp1(fm,Bandset2_HM(:,1),fm_p,method); interp1(fm,Bandset2_HM(:,2),fm_p,method)]';       
Bandset2_HM = Bandset2_HM_p;
Bandset2_HM(:,1) = Bandset2_HM(:,1)./ OmegaRset2(:,1);
Bandset2_HM(:,2) = Bandset2_HM(:,2)./ OmegaRset2(:,1);  

W2_HM = Bandset2_HM(:,2) - Bandset2_HM(:,1);

% mu_D versus alpha (K/E=0.02, rrho=3, 5, 6, 8, 11, 15, 17, 20,)
method = 'spline';
Rt =[1/3 2/3 3/3];         % t/r;
t = Rt*r;
cAlpha = R./sqrt(R^2+t.^2);
Geom = Rt.*cAlpha;
Omega_R1 = [4.9796e3
            3.8572e3
            3.5211e3
            3.0494e3
            2.6005e3
            2.2269e3
            2.0918e3
            1.9286e3
            ];
Omega_R1=[interp1(fm,Omega_R1,fm_p,method)]';

mu_D1 = [0.4199
         0.5469
         0.5916
         0.6590
         0.7266
         0.7837
         0.8042
         0.8285]*100;
rmu_D1=[interp1(fm,mu_D1,fm_p,method)]'./fm_p';   % mu_D/mu_rho

Omega_R2 = [9.4291e3
            7.3038e3
            6.6675e3
            5.7742e3
            4.9243e3
            4.2169e3
            3.9611e3
            3.6520e3];
Omega_R2=[interp1(fm,Omega_R2,fm_p,method)]';

mu_D2 = [0.1524
         0.2311
         0.2652
         0.3251
         0.3986
         0.4748
         0.5061
         0.5467]*100;
rmu_D2=[interp1(fm,mu_D2,fm_p,method)]'./fm_p';

Omega_R3 = [1.3061e4
            1.0117e4
            9.2356e3
            7.9983e3
            6.8211e3
            5.8412e3
            5.4869e3
            5.0587e3];
Omega_R3=[interp1(fm,Omega_R3,fm_p,method)]';        
        
mu_D3 = [0.0735
        0.1174
        0.1378
        0.1758
        0.2271
        0.2862
        0.3125
        0.3485
        ]*100;
    
rmu_D3 = [interp1(fm,mu_D3,fm_p,method)]'./fm_p';

Omega_R = [Omega_R1 Omega_R2 Omega_R3]/1000;
rmu_D = [rmu_D1 rmu_D2 rmu_D3];
fm = fm_p;    
%% Plots

Dev1 = abs(W1_BA - W1_HM)./W1_BA * 100;      % deviation (>20% after 18th)
Dev2 = abs(W2_BA - W2_HM)./W2_BA * 100;      % deviation (>20% before 3th)

% comparison 1
figure
subplot(1,3,1)
hold on

[AX,H1,H2]=plotyy(rK,W1_BA,rK,OmegaUset1./OmegaRset1(:,1));
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);

plot(rK,W1_HM,'-ksquare','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',10)

set(AX(1),'xscale', 'log','FontSize',16,'FontName','Times New Roman')
set(AX(2),'xscale', 'log','FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','K/E','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
legend('W_b/\omega_{res}-BA','W_b/\omega_{res}-HM','\mu_{\omega}')

subplot(1,3,2)
hold on
[AX,H1,H2]=plotyy(fm,W2_BA,fm,OmegaUset2./OmegaRset2(:,1));
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);

plot(fm,W2_HM,'-ksquare','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','r','MarkerSize',10)

set(AX(1),'FontSize',16,'FontName','Times New Roman')
set(AX(2),'FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','\mu_\rho [%]','FontName','Times New Roman','FontWeight','Bold','FontSize',16);

subplot(1,3,3)
hold on
[AX,H1,H2]=plotyy(fm,rmu_D,fm,Omega_R);
set(H1,'color','k','marker','square','LineStyle','-','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','b','MarkerSize',10);
set(H2,'color','k','marker','o','LineStyle',':','linewidth',2,'MarkerEdgeColor','none','MarkerFaceColor','k','MarkerSize',10);
set(AX(1),'FontSize',16,'FontName','Times New Roman')
set(AX(2),'FontSize',16,'FontName','Times New Roman')
set(get(AX(1),'Xlabel'),'String','\mu_\rho [%]','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
set(get(AX(1),'Ylabel'),'String','\mu_D/\mu_\rho','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
set(get(AX(2),'Ylabel'),'String','\omega_{res} [kHz]','FontName','Times New Roman','FontWeight','Bold','FontSize',16);
legend('f_{geo}=0.326','f_{geo}=0.619','f_{geo}=0.858')
set(gcf,'Units','centimeters','Position',[5 0 35 15])