%-------------------------------------------------------------------------%
% Background
% Computational homogenization under the periodic boundary condition
% Created by Lei, TU/e, 2017
%-------------------------------------------------------------------------%

function [CMO,DMO,rhoM,H,jh,NormkH, FreqH,DynParas,CS,DS,rhoS] = compHom(W,V,coords,coordsRef,presNodes,Kqs,Mqs,Mff,Mpf,S,Nset,Omega,Fai,DirWave)

presDof = 2*length(presNodes);
nModes = Nset(1);
nSamples = Nset(2);
Nomega = Nset(3);

% effective tangent and inertial coupling
[CMO,DMO,presCoords]= getCD(coords,presNodes,coordsRef,Kqs,Mqs);
CMO = CMO/V;
DMO = DMO/V;

% effective density
Ip = ones(presDof,1);
rhoM = zeros(2,2);
rhoM(1,1) = Ip(1:2:end)'*Mqs(1:2:end,1:2:end)*Ip(1:2:end);
rhoM(2,2) = Ip(2:2:end)'*Mqs(2:2:end,2:2:end)*Ip(2:2:end);
rhoM(1,2) = Ip(1:2:end)'*Mqs(1:2:end,2:2:end)*Ip(2:2:end);
rhoM(2,1) = Ip(2:2:end)'*Mqs(2:2:end,1:2:end)*Ip(1:2:end);
rhoM = rhoM/V;

% collect coupling coefficients H and j
H = cell(nModes,1);
jh = zeros(nModes,2);

for iMode = 1:nModes
    
    % current mode
    fai = Fai(:,iMode);
    
    % linear superposition ([uf; up] = [fai,S;0 I]*[eta;up])
    % reduce the coupling system (Kqs*up + Mqs*ap + mc*ddeta = fiextEffp)
    mc = S'*Mff*fai + eye(presDof)*Mpf*fai;
    
    % allocate H component
    h = zeros(2,2);
    for ip = 1:length(presNodes)
        h = h + mc(2*ip-1:2*ip)*presCoords(ip,:);
        
    end
    H{iMode} = h;
    
    % allocate j component
    jh(iMode,1) = Ip(1:2:end,1)'* mc(1:2:end);
    jh(iMode,2) = Ip(2:2:end,1)'* mc(2:2:end);
    
end

% get dispersion curves
[NormkH, FreqH,DynParas,CS,DS,rhoS] = disperHom(W,DirWave,CMO,DMO,rhoM,H,jh,Omega,Nomega);  

end