%-------------------------------------------------------------------------%
% Background
% Function to do dispersion analysis using homogenized model
% Created by Lei, TU/e, 2017
%-------------------------------------------------------------------------%
function [NormkH, FreqH, DynParas,CS,DS,rhoS] = disperHom(W,DirWave,CMO,DMO,rhoM,H,jh,Omega,Nomega)

% initialize the frequency axis (Hz)
Lomega = 0;
% Uomega =2*max(Omega);
Uomega = Omega(3)*2;
omega = linspace(Lomega,Uomega,Nomega);

% store the dynamic parameters, rho, K
DynParas.omega = omega;
DynParas.rho = zeros(4,length(omega));
DynParas.C = zeros(4,length(omega));

% compute Hw
Hh = zeros(size(jh));
for io = 1:length(Omega)

    Hh(io,1) = DirWave(1) * H{io}(1,1) + DirWave(2) * H{io}(1,2);
    Hh(io,2) = DirWave(1) * H{io}(2,1) + DirWave(2) * H{io}(2,2);
     
end


% Compute C_theta and D_theta
Ct = zeros(2,2);
Dt = zeros(2,2);

for Nrow =1:4
    
    % determine i
    i = 1*(Nrow == 1 || Nrow ==3) + 2*(Nrow == 2 || Nrow ==4);
    
    % determine j
    j = 2*(Nrow == 2 || Nrow ==3) + 1*(Nrow == 1 || Nrow ==4);
    
    for Ncol = 1:4
        % determine k
        kr = 2*(Ncol == 2 || Ncol == 3) + 1*((Ncol == 1 || Ncol == 4));
        
        % determine l
        l = 1*(Ncol == 1 || Ncol == 3) + 2*(Ncol == 2 || Ncol == 4);
        
        Ct(j,kr) = DirWave(i)*CMO(Nrow,Ncol)* DirWave(l) + Ct(j,kr);
        Dt(j,kr) = DirWave(i)*DMO(Nrow,Ncol)* DirWave(l) + Dt(j,kr);
        
    end
end

% static part
rhoS = rhoM;
CS = Ct;
DS = Dt;

% Loop for each frequency (real real imag imag)
NormkH = zeros(4,length(omega));
FreqH = zeros(4,length(omega));
for i =1:length(omega)
    
    % transfer frequency to angular frequency 
    iOmega = omega(i) * 2*pi;
    DynParas.omega(i) = iOmega;
    rOmega = Omega * 2*pi;          % resonant frequency
    
    % compute the dynamic density
    rhoE = zeros(2,2);                       % deviatoric
    CE = zeros(2,2);
    for s =1:length(rOmega)  
        
        J = [jh(s,1)*jh(s,1) jh(s,1)*jh(s,2);...
              jh(s,2)*jh(s,1) jh(s,2)*jh(s,2) ]/W/W;
        H = [Hh(s,1)*Hh(s,1) Hh(s,1)*Hh(s,2);...
              Hh(s,2)*Hh(s,1) Hh(s,2)*Hh(s,2) ]/W/W;
          
        rhoE = rhoE + iOmega^2 / (rOmega(s)^2 - iOmega^2)*J;
        CE = CE + iOmega^4 / (rOmega(s)^2 - iOmega^2)*H;

    end

    rhoD = rhoS*eye(2)+ rhoE;             % dynamic density
    CD = (CS - iOmega^2 * DS)*eye(2) - CE;                  % dynamic stiffness
    
    DynParas.rho(:,i) = [rhoD(1,1);rhoD(2,2);rhoD(1,2);rhoD(2,1)];
    DynParas.C(:,i) = [CD(1,1);CD(2,2);CD(1,2);CD(2,1)];

    % eigenvalue analysis (two components for k)
    [Uk,k2] = eig(rhoD*iOmega^2,CD);
   
    % real part
    kr = real(sqrt(diag(k2)))*W/pi;
    
    for j=1:2
    NormkH(j,i) = kr(j);
    FreqH(j,i) = omega(i);
    
    if NormkH(j,i)>1 ||  NormkH(j,i)<=0
        FreqH(j,i) = nan;
    end
    end
    
    % imagninary part
    ki = imag(sqrt(diag(k2)))*W/pi;
    
    for j=3:4
    NormkH(j,i) = -ki(j-2);
    FreqH(j,i) = omega(i);
    
    if NormkH(j,i)<-1 ||  NormkH(j,i)>=0
        FreqH(j,i) = nan;
    end
    end
    
end
