%-------------------------------------------------------------------------%
% Background
% Convert the mesh file from Gmsh to taht can be used for homogenization
% only one type of mateiral
% 2D
% Created by Lei, TU/e, 2017
%-------------------------------------------------------------------------%

clear, clc

outputGmsh = ['Meshes/Matrix2D_mapping.msh'];      % mesh file exported from Gmsh
meshM = ['Meshes/Matrix2D_mapping.txt'];  % mesh file required

nElemNodes = 4;       % element node number
ElemMarker = 3;       % 3: quadrilateral element


%% Read mesh data

fgmsh = fopen(outputGmsh,'r');
nStrings = 0;         % mark the names, 1-MeshFormat, 3-Nodes, 5-Elements
coords = [];
elems = [];
nNodeCoords = 2;      % 2D

while ~feof(fgmsh)
    
    % read one line 
    tline = fgetl(fgmsh);
    ltline = length(tline);
    numtline = str2num(tline);
    
    % get the names
    if isempty(numtline) && ~isempty(tline)
        
        % detect the mesh format
        if ltline>=11 && strcmpi(tline(1:11),'$MeshFormat')
            nStrings = nStrings+1;
        end
        
        if ltline>=14 && strcmpi(tline(1:14),'$EndMeshFormat')
            nStrings = nStrings+1;
        end
        
        % detect the nodes
        if ltline>=6 && strcmpi(tline(1:6),'$Nodes')
            nStrings = nStrings+1;
        end
        
        if ltline>=9 && strcmpi(tline(1:9),'$EndNodes')
            nStrings = nStrings+1;
        end
        
        % detect the elements
        if ltline>=9 && strcmpi(tline(1:9),'$Elements')
            nStrings = nStrings+1;
        end
        
        if ltline>=12 && strcmpi(tline(1:12),'$EndElements')
            nStrings = nStrings+1;
        end
    end   
     
     % scan the data
     if ~isempty(numtline)  
            
         % read the nodes
         if nStrings == 3 && length(numtline)>1
                
             coords = [coords; numtline(2:end-1)];
         end
            
         % read the elements
         if nStrings == 5 && length(numtline)>1 && numtline(2) == ElemMarker
             elems = [elems; numtline(end-(nElemNodes-1):end)];
         end
            
     end
        
end

fclose(fgmsh);


%% Plot and confirm the mesh (only for 2D mesh, 3D mesh is too time-consuming)
nNodes = size(coords,1);
nElems = size(elems,1);

X0 = min(coords(:,1));
Y0 = min(coords(:,2));
    
L = max(coords(:,1)) - X0;
 H = max(coords(:,2)) - Y0;
    
figure
hold on
rectangle('Position', [0 0 L H], 'facecolor', 'k', 'edgecolor','none' );
    
for iElem =1:nElems
     elemNodes = elems(iElem,:);
     nodeZone = [elemNodes elemNodes(1)];
     fill(coords(nodeZone,1),coords(nodeZone,2),[0.25 0.25 0.25],'edgecolor','w','linewidth',0.5);
        
end
    
hold off
xlim([X0 X0+L]);
ylim([Y0 Y0+H]);
axis('equal')
axis off
set(gca,'xtick',[])
set(gca,'ytick',[])
box on
    
title('Mesh','FontName','Times New Roman','FontWeight','Bold','FontSize',16,'interpreter','latex')
set(gcf,'Units','centimeters','Position',[5 5 20 15]);
    

%% Write a mesh file for LS-DYNA

flsd = fopen(meshM,'wt');

% nodes
fprintf(flsd,'NNODES ');
fprintf(flsd,'%i\n',size(coords,1));
for iNode = 1:nNodes
    fprintf(flsd,'%8i%16.7f%16.7f\n',iNode-1,coords(iNode,:));
    
end

% elements
fprintf(flsd,'\n');
fprintf(flsd,'NELEMS ');
fprintf(flsd,'%i\n',size(elems,1));
for iElem = 1:nElems
    fprintf(flsd,'%8i%8i%8i%8i%8i\n',iElem-1,elems(iElem,:)-1);
    
end

% corners
fprintf(flsd,'\n');
fprintf(flsd,'CORNER\n');
fprintf(flsd,'%i %i %i %i\n',[0 1 3 2]);

% boundaries
fprintf(flsd,'\n');
fprintf(flsd,'LEFT\n');
index1 = find(coords(:,1)==0);
index2 = find(coords(:,2)~=0 & coords(:,2)~=1);
indexL = intersect(index1,index2);
coordL = coords(indexL,:);
[~,index] = sort(coordL(:,2));
indexL = indexL(index)-1;
fprintf(flsd,'%8i',indexL);
fprintf(flsd,'\n');

fprintf(flsd,'\n');
fprintf(flsd,'RIGHT\n');
index1 = find(coords(:,1)==1);
index2 = find(coords(:,2)~=0 & coords(:,2)~=1);
indexR = intersect(index1,index2);
coordR = coords(indexR,:);
[~,index] = sort(coordR(:,2));
indexR = indexR(index)-1;
fprintf(flsd,'%8i',indexR);
fprintf(flsd,'\n');

fprintf(flsd,'\n');
fprintf(flsd,'BOTTOM\n');
index1 = find(coords(:,2)==0);
index2 = find(coords(:,1)~=0 & coords(:,1)~=1);
indexB = intersect(index1,index2);
coordB = coords(indexB,:);
[~,index] = sort(coordB(:,1));
indexB = indexB(index)-1;
fprintf(flsd,'%8i',indexB);
fprintf(flsd,'\n');

fprintf(flsd,'\n');
fprintf(flsd,'TOP\n');
index1 = find(coords(:,2)==1);
index2 = find(coords(:,1)~=0 & coords(:,1)~=1);
indexT = intersect(index1,index2);
coordT = coords(indexT,:);
[~,index] = sort(coordT(:,1));
indexT = indexT(index)-1;
fprintf(flsd,'%8i',indexT);

fclose(flsd);




