%-------------------------------------------------------------------------%
% Background
% Function to get system stiffness matrix and mass matrix
% Created by Lei, TU/e, 2017
%-------------------------------------------------------------------------%

function [K,M] = assembleKeMe(coords,elems,CInt,rho,nIntElem,dVElem,order)

% initialize
totDof = 2*length(coords);
K = zeros(totDof,totDof);
M = zeros(totDof,totDof);

% assemble
for ielem =1:length(elems)
    
    elemNode = elems(ielem,:);
    elemDofs = getDofs(elemNode);
    
    % shape function information
    sData = getElemShapeData(coords(elemNode,:),order);

    for is =1:length(sData)
        iData = sData{is};
        C = CInt{nIntElem*(ielem-1)+is};
        
        [B,H]= getBHmatrix(iData.dhdx,iData.h);
        
        % K
        Ke = B'*(C*B) * iData.weight * dVElem;
        K(elemDofs,elemDofs) = K(elemDofs,elemDofs)+Ke;
        
        % M
        Me = H'*H*rho * iData.weight * dVElem;
        M(elemDofs,elemDofs) = M(elemDofs,elemDofs)+Me; 

    end

end
